#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['fractal',
 'fractal.contrib',
 'fractal.contrib.django',
 'fractal.contrib.fastapi',
 'fractal.contrib.fastapi.exceptions',
 'fractal.contrib.fastapi.routers',
 'fractal.contrib.fastapi.routers.domain',
 'fractal.contrib.fastapi.utils',
 'fractal.contrib.mongo',
 'fractal.contrib.roles',
 'fractal.contrib.tokens',
 'fractal.core',
 'fractal.core.command_bus',
 'fractal.core.event_sourcing',
 'fractal.core.event_sourcing.projectors',
 'fractal.core.exceptions',
 'fractal.core.models',
 'fractal.core.process',
 'fractal.core.process.actions',
 'fractal.core.repositories',
 'fractal.core.services',
 'fractal.core.specifications',
 'fractal.core.specifications.generic',
 'fractal.core.utils']

package_data = \
{'': ['*']}

install_requires = \
['python-dotenv']

extras_require = \
{'test': ['black', 'flake8', 'isort']}

setup(name='fractal-toolkit',
      version='0.1.8',
      description='Fractal is a scaffolding toolkit for building SOLID logic for your Python applications.',
      author='Douwe van der Meij',
      author_email='douwe@karibu-online.nl',
      url='https://github.com/douwevandermeij/fractal',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.8',
     )
